'''
  Copyleft ITEG IT-Engineers GmbH 2022
'''

import base64
import configparser
#import logging
import os
import sys

#from clazzes.datetools import import to_datetime_timestamp_millis, 
#from datetime import datetime

from clazzes.gitlabclient import glclogging
#from clazzes.gitlabclient.helpers import formatFileSize
#from array import array


log = glclogging.getLogger()


def getGLCConfiguration():
    if GLCConfiguration.staticGLCConfiguration == None:
        GLCConfiguration.staticGLCConfiguration = GLCConfiguration()
    return GLCConfiguration.staticGLCConfiguration


class GLCConfiguration:
    '''
      holds eventual parsed configuration and retrieved info about a known gitlab system
    '''

    # static holder for first instance
    staticGLCConfiguration = None
    
    def __init__(self, cnfFileName:str=""):
        userCnfFileName = os.path.expanduser("~/.gitlab-client/gitlab-client.cnf")
        if os.path.exists(userCnfFileName):
            self.parseConfiguration(cnfFileName = userCnfFileName)
        elif os.path.exists("/etc/gitlab-client/gitlab-client.cnf"):
            self.parseConfiguration("/etc/gitlab-client/gitlab-client.cnf")
        else:
            print("ERROR: No ~/.gitlab-client/gitlab-client.cnf or /etc/gitlab-client/gitlab-client.cnf found, anything but --help will fail", file=sys.stderr)
            self.parseConfiguration("/etc/gitlab-client/gitlab-client.cnf") # call anyway, for sane defaults
        GLCConfiguration.staticGLCConfiguration = self

    def parseConfiguration(self, cnfFileName = ""):
        
        self.conf = configparser.ConfigParser(allow_no_value=True)
        self.conf.optionxform = lambda option: option
        
        # collection of GitlabInfo objects, see class below
        self.gitlabInfos = {}
        self.primaryGitlabName = "";
    
        if os.path.exists(cnfFileName):
            log.debug("parseGLCConfiguration(): Loading config file [%s]." % cnfFileName)
            self.conf.read(cnfFileName,'utf-8')
                
            for secName in self.conf.sections():
                if secName == 'global':
                    verbosity = self.conf["global"].get('verbosity')
                    if verbosity == None:
                        verbosity = 1
                    if int(verbosity) == None:
                        verbosity = 1
                    if int(verbosity) > log.verbosity:
                        log.verbosity = int(verbosity)
                    continue
            
                gitlabInfo = GitlabInfo(gitlabName = secName, conf = self.conf)
                self.gitlabInfos[secName] = gitlabInfo
                log.debug("parseGLCConfiguration(): Detected configured Gitlab instance %s:\n%s" % (
                    secName, gitlabInfo.describeValues(indent="  ")))
                   
                if len(self.primaryGitlabName) == 0:
                    self.primaryGitlabName = secName
    
        if len(self.gitlabInfos) == 0:
            log.error("No Gitlab instances configured in any gitlab-client.cnf\n")
            #sys.exit(1)
        
        log.debug("parseGLCConfiguration(): Detected %d configured Gitlab systems, primary=%s." % (
            len(self.gitlabInfos), self.primaryGitlabName))


class GitlabInfo:
    '''
      holds configured and retrieved info about a known gitlab system
    '''

    def __init__(self, gitlabName: str, conf):
        # configured stuff
        self.name = gitlabName
        self.api_url = conf[gitlabName].get('api_url')
        self.api_token = conf[gitlabName].get('api_token', "-")
        # see https://docs.gitlab.com/ee/api/#authentication
        self.api_auth_headername = "Authorization"
        self.api_auth_headervalue = "Bearer " + self.api_token
        self.glc_systeminfo = None
        self.projectInfos = None
                                
    def describeValues(self, indent="  "):
        desc = ""
        desc += indent + "Gitlab name: %s" % self.name + "\n"
        desc += indent + "  API URL: %s" % self.api_url + "\n"
        #desc += indent + "  API User: %s" % self.api_user + "\n"
        #if len(self.api_password) > 1:
        #    pw_obfuscated = self.api_password[:2] + "..." + self.api_password[-2:] 
        #    desc += indent + "  API Password: %s\n" % pw_obfuscated
        #else:
        #    desc += indent + "  API Password: -\n"
        if len(self.api_token) > 1:
            token_obfuscated = self.api_token[:2] + "..." + self.api_token[-2:] 
            desc += indent + "  API Access Token: %s\n" % token_obfuscated
        else:
            desc += indent + "  API Access Token: -\n"
        # + "\n"
        return desc


class GitlabSystemInfo:
    '''
      holds info retrieved via API call /versions 
    '''

    def __init__(self, sysinfo: {}):
        self.version = sysinfo["version"]
        self.revision = sysinfo["revision"]

    def describeValues(self, indent="  "):
        desc = ""
        desc += indent + "Gitlab System Info:\n"
        desc += indent + "  Version: %s" % self.version + "\n"
        desc += indent + "  Revision: %s" % self.revision + "\n"
        return desc


class GitlabProjectInfo:
    '''
      holds info retrieved via API call /projects 
    '''

    def __init__(self, projectDict: {}):
        self.id = projectDict["id"]
        self.description = projectDict["description"]
        self.default_branch = projectDict["default_branch"]
        self.ssh_url = projectDict["ssh_url_to_repo"]
        self.web_url = projectDict["web_url"]
        self.name_with_namespace = projectDict["name_with_namespace"]
        self.path_with_namespace = projectDict["path_with_namespace"]
        self.created_at = projectDict["created_at"]
        self.last_activity_at = projectDict["last_activity_at"]

    def describeValues(self, indent="  "):
        desc = ""
        desc += indent + "Project %s:\n" % self.path_with_namespace
        desc += indent + "  Name: %s\n" % self.name_with_namespace
        desc += indent + "  Web URL: %s\n" % self.web_url
        desc += indent + "  SSH URL: %s\n" % self.ssh_url
        return desc

    



