'''
  $Id$
  
  Copyleft ITEG IT-Engineers GmbH 2021
'''

import getopt
#import logging
import os
import subprocess
import sys

from clazzes.gitlabclient import glclogging
from clazzes.gitlabclient.dto import GLCConfiguration
from clazzes.gitlabclient.restclient import GLCRestClient

log = glclogging.getLogger()
glcConfiguration = None


def describeUsage(argv0):
    usage = ""
    usage += "\n"
    usage += "Usage: %s [options...] [projectFilter]\n"%argv0
    usage += "\n"
    usage += "Clones all projects matching the filter in the current directory\n"
    usage += "after checking back with a list of projects that would be clone.\n"
    usage += "\n"
    usage += "An eventual projectFilter is applied to the projects' path_with_namespace\n"
    usage += glclogging.getCommonUsageOptions()
    usage += "  Specific options are:\n"
    usage += "    -n gitlabName  Use Gitlab instance with this section name in gitlab-client.cnf\n"
    usage += "    -p             Pull projects that are already cloned\n"
    usage += "    -y             Clone projects without checking back\n"
    return usage


def run_glc_projects_clone(argv0,args):
    
    try:
        opts, projectFilters = getopt.getopt(args, "dhn:pqtvy", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    glcConfiguration = GLCConfiguration()
    restclient = GLCRestClient(glcConfiguration)
    
    gitlabName = ""
    projectFilter = ""
    yessed = False
    pullPreClonedProjects = False

    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        elif o in ("-n"):
            gitlabName = a
        elif o in ("-p"):
            pullPreClonedProjects = True
        elif o in ("-y"):
            yessed = True
        else:
            log.warning("Unhandled option: %s %s" % (o, a))
            
    if len(projectFilters) == 0:
        log.warning("No projectFilter specified, going to clone really all.")
    elif len(projectFilters) == 1:
        projectFilter = projectFilters[0]
    else:
        log.error("Multiple projectFilters specified, this is not supported.")
        sys.exit(1)
    
    # single Gitlab only    
    if len(gitlabName) == 0:
        gitlabName = glcConfiguration.primaryGitlabName
        log.debug("Using primary Gitlab instance, %s" % gitlabName)

    gitlabInfo = glcConfiguration.gitlabInfos.get(gitlabName)
    
    if gitlabInfo == None:
        log.error("Unkown Gitlab instance name: '%s'" %
              gitlabName)
        sys.exit(1)
        
    restclient.fetchProjectList(gitlabInfo=gitlabInfo, projectFilter=projectFilter)

    nrProjects = len(gitlabInfo.projectInfos)

    if len(projectFilter) == 0:
        log.p("\nGitlab %s has %d projects accessible by you:\n" % (
            gitlabName, nrProjects))
    else:
        log.p("\nGitlab %s has %d projects accessible by you matching the specified filter:\n" % (
            gitlabName, nrProjects))
    for projectPath in sorted(gitlabInfo.projectInfos.keys()):
        projectInfo = gitlabInfo.projectInfos[projectPath]
        log.p(projectInfo.describeValues(indent="  "))

    if not yessed:
        choice = input("OK to clone %d projects under current directory?\n"
                       "If your are You sure it is ok to continue, enter 'yes' ... " % nrProjects)
        if choice != "yes":
            log.error("You chose to abort.")
            sys.exit(1)

    log.info("Actually going to clone %d projects" % nrProjects)

    cloneBaseDir = os.getcwd()
    
    nrProjectsCloned = 0
    nrProjectsPulled = 0
    nrProjectsSkipped = 0
    
    for projectPath in sorted(gitlabInfo.projectInfos.keys()):
        projectInfo = gitlabInfo.projectInfos[projectPath]
        projectFullPath = cloneBaseDir + "/" + projectPath
        if os.path.exists(projectFullPath):
            if pullPreClonedProjects:
                if os.path.isdir(projectFullPath+"/.git"):
                    log.info("Not re-cloning but pulling project in existing project path %s" % projectFullPath)
                    os.chdir(projectFullPath)
                    cp = subprocess.run(["/usr/bin/git", "pull"])
                    if cp.returncode != 0:
                        log.error("Failed to git pull in %s" % projectFullPath)
                        sys.exit(1) # or continue?
                    log.good("Successfully pulled in %s" % projectFullPath)
                    nrProjectsPulled += 1
                else:
                    log.info("Not re-cloning project into existing non-project path %s" % projectFullPath)
                    nrProjectsSkipped += 1
            else:
                log.info("Not re-cloning project into existing path %s" % projectFullPath)
                nrProjectsSkipped += 1
            continue
        log.info("Processing project with path %s" % projectPath)
        projectGroupDirPath = os.path.dirname(projectFullPath)
        if not os.path.exists(projectGroupDirPath):
            log.info("Auto-creating group directory %s" % projectGroupDirPath)
            os.makedirs(projectGroupDirPath, exist_ok=False) # mode = 0o770
        log.info("Entering group directory %s" % projectGroupDirPath)
        log.info("Calling git clone %s" % projectInfo.ssh_url)
        os.chdir(projectGroupDirPath)
        cp = subprocess.run(["/usr/bin/git", "clone", projectInfo.ssh_url])
        if cp.returncode != 0:
            log.error("Failed to git clone %s" % projectInfo.ssh_url)
            sys.exit(1) # or continue?
        log.good("Successfully cloned %s to %s" % (projectInfo.ssh_url, projectFullPath))
        nrProjectsCloned += 1

    log.good("\nSuccessfully processed %s projects. Skipped %d, pulled %d and cloned %d projects to group directories under %s\n" % (
             nrProjects, nrProjectsSkipped, nrProjectsPulled, nrProjectsCloned, cloneBaseDir))
    
    os.chdir(cloneBaseDir)

    sys.exit(0)


